<?php

/*
  Scriptol XML to array extension to PHP
  (c) 2014-2015 Scriptol.org
  License: MIT.
*/

// Load an XML file into an array

function scriptol_unxml(&$X) 
{  
    $att = array();
    foreach($X as $tag => $obj)
    {
        if(!is_array($obj)) {
            $X[$tag] = array();
            $X[$tag]['data'] = $obj;
            continue;
        }
        if($tag==='@attributes') {
            foreach($obj as $k => $v)
                $att[$k]=$v;  
            unset($X['@attributes']);
            continue;
        }
        scriptol_unxml($X[$tag]);
    }
    foreach($att as $k=>$v) // must be delayed
        $X[$k]=$v;
    return;
}


function scriptol_loadXML(&$d, $fname) {
    $xmlroot = array();
    $xml = simplexml_load_file($fname); 
    $xmlroot[$xml->getName()] = json_decode(json_encode((array) $xml), 1);
    scriptol_unxml($xmlroot);
    $d = $xmlroot;
}

function scriptol_dload(&$d, $fname, $isxml=false) 
{
    $infos = pathinfo($fname);    
    if(isset($isxml) && $isxml ||  in_array($infos['extension'], ["xml","rss","svg"])) {
        scriptol_loadXML($d, $fname);
        return;
    }
    $d = file($fname);    
}


function scriptol_xmlsub($xml,$array)
{
  $params=array();
  foreach($array as $k=>$v)
  {
    if(is_array($v))
        scriptol_xmlsub($xml->addChild($k), $v);
    else {
     if($k == "data")
      $xml->{0} = $v;
     else
      $xml->addAttribute($k,$v);
    }
  }
}


// Convert an array to a XML document in a string.

function scriptol_toXML($array)
{
  $xml=new SimpleXMLElement('<?xml version=\'1.0\' encoding=\'utf-8\'?'.'><'.key($array).'/>');
  scriptol_xmlsub($xml,$array[key($array)]);
  return $xml->asXML();
} 


function scriptol_byNamesSub($d, $name, &$tlist) {
  foreach($d as $k) 
  {
    if($k == $name) $tlist.push($d);
    $t = gettype($d[$k]);
    if($t === "object" || $t ==="array") scriptol_byNamesSub($d[$k], $name, $tlist);
  }
  return;
}

function scriptol_getByNames($d, $name) {
  $tlist = [];
  scriptol_byNamesSub($d, $name, $tlist);
  return $tlist;  
}

function scriptol_getById($d, $idval) {
  foreach($d as $k) {
    $t = gettype($d[$k]);
    if($t === "object" || $t === "array") {
      $dsub = $d[$k];
      foreach($dsub as $k => $v)
      {
        if("id" == $k && $dsub[$id] == $idval) return $d;
      }  
      $dret = scriptol_getById($dsub, $idval);
      if($dret) return $dret;
    }
  }
  return false;
}



function scriptol_setById($d, $idval, $subdict) {
  foreach($d as $k) {
    $t = gettype($d[$k]);
    if($t === "object" || $t === "array") {
      $dsub = $d[$k];
      foreach($dsub as $k => $v)
      {
        if("id" == $k && $dsub[$id] == $idval) {
          $d[$k]=$subdict[$k];
          return true;
        }
      } 
      if (scriptol_setById($dsub, $idval, $subdict)) return true;
    }
  }
  return false;
}


function scriptol_updByIdSub($dold, $dnew) {
  foreach($dnew as $k) 
  {
    if(gettype($dnew[$k]) === 'object') 
    {
      if(gettype($dold[$k]) !== 'object') 
      {
        $dold[$k] = array();
      }  
      $dold[$k] = scriptol_updByIdSub($dold[$k], $dnew[$k]);
    }  
    else
      $dold[$k] = $dnew[$k];
  }
  return $dold;
}

function scriptol_updById($d, $idval, $subdict) {
  $dfound = getById($d, $idval);
  if($dfound) {
    $dfound = scriptol_updByIdSub($dfound, $subdict);    
  }
  setById($d, $idval, $dfound);
}


?>
